<?php

namespace FileRun\Files;

use FileRun\Perms;
use FileRun\Versions;
use FileRun\Users;
use FileRun\Log;
use FileRun\SearchIndexQueue;
use FileRun\Share;
use FileRun\Stars;
use FileRun\MetaFiles;
use FileRun\MetaTypes;
use FileRun\Paths;
use FileRun\Thumbs;

class Manager {

	var $error;
	var $uid;

	function setUID($uid) {
		$this->uid = $uid;
	}
	function loggedUID() {
		global $auth;
		return ($this->uid == $auth->currentUserInfo['id']);
	}
	function getHomeFolder($uid = false) {
		if (!$uid) {
			if ($this->uid) {
				$uid = $this->uid;
			} else {
				global $auth;
				$uid = $auth->currentUserInfo['id'];
			}
		}
		return Perms::getOne('homefolder', $uid);
	}
	function getUserAbsolutePath($path, $uid = false) {
		$pathInfo = Utils::parsePath($path);
		if ($pathInfo['share_id']) {//shared folder
			$shareInfo = Share::getById($pathInfo['share_id']);
			if (!$shareInfo) {return false;}
			$fullPath = $shareInfo['path'];
			if (!$fullPath) {return false;}
			if ($pathInfo['relative_path']) {
				$fullPath = gluePath($fullPath, $pathInfo['relative_path']);
			}
		} else {
			if ($pathInfo['trash_id']) {
				$fullPath = \FileRun::getTrash()->getTrashPath($pathInfo['trash_id']);
			} else {
				$fullPath = $this->getHomeFolder($uid);
				$fullPath = gluePath($fullPath, str_replace("/ROOT/HOME", "", $path));
			}
		}
		return $fullPath;
	}

	function isValidRelativePath($path) {
		return Utils::isCleanPath($path);
	}

	function deleteFile($relativePath, $filename, $permanent = false) {
		if (!$this->isValidRelativePath($relativePath)) {
			return false;
		}

		$path = $this->getUserAbsolutePath($relativePath);
		$filePath = gluePath($path, $filename);
		if (!is_file($filePath)) {
			$this->error = array("code" => 2, "msg" => "Invalid path, not a file.");
			return false;
		}

		if (Versions::isLockedBySomebodyElse($filePath)) {//check locking
			$this->error = array("code" => 3, "msg" => "File is locked by somebody else.");
			return false;
		}

		$hrs = \FileRun::hook('file.delete', func_get_args(), array('filePath' => $filePath));
		if (is_array($hrs)) {
			$this->error = $hrs['error'];
			return $hrs['return'];
		}

		if ($permanent) {
			$rs = unlink($filePath); //delete for good
		} else {
			$trash = \FileRun::getTrash();
			$rs = $trash->store(gluePath($relativePath, $filename));
		}

		if ($rs) {
			if ($permanent) {
				self::clearFileData($filePath);
			} else {
				\FileRun::getWebLinks()->deleteByPath($filePath);
				Stars::removeByPath($filePath);
				//move data to $trash->lastStoredPath
				Versions::move($filePath, $trash->lastStoredPath);//move versions too
				Thumbs\Utils::clearCacheItem($filePath);
				Share::clear($filePath, false);
				Paths::updatePath($filePath, $trash->lastStoredPath);//clear paths
				SearchIndexQueue::add($filePath, "delete");
			}

			Log::add($uid = false, "file_deleted", array(
				"full_path" => $filePath,
				"relative_path" => gluePath($relativePath, $filename),
				"trash_full_path" => $trash->lastStoredPath,
				"permanent" => $permanent
			));
		} else {
			$this->error = $trash->error;
			Log::add($uid = false, "file_deletion_failed", array(
				"full_path" => $filePath,
				"relative_path" => gluePath($relativePath, $filename),
				"permanent" => $permanent,
				"error" => $trash->error
			));
		}
		return $rs;
	}

	static function clearFileData($path) {
		Thumbs\Utils::clearCacheItem($path);
		Versions::clear($path);
		SearchIndexQueue::add($path, "delete");

		Share::clear($path, false);
		\FileRun::getWebLinks()->deleteByPath($path);
		Stars::removeByPath($path);

		MetaFiles::deleteByPath($path);
		Paths::deletePath($path);
	}

	function deleteFolder($relativePath, $permanent = false) {
		global $fm;
		if (!$this->isValidRelativePath($relativePath)) {
			return false;
		}

		$path = $this->getUserAbsolutePath($relativePath);
		if (!is_dir($path)) {
			$this->error = array("code" => 4, "msg" => "Invalid path, not a folder.");
			return false;
		}

		if (strtolower(gluePath($this->getHomeFolder(), "/")) == strtolower(gluePath($path))) {
			$this->error = array("code" => 5, "msg" => "You are not allowed to delete your home folder.");
			return false;
		}

		$hrs = \FileRun::hook('folder.delete', func_get_args(), array('path' => $path));
		if (is_array($hrs)) {
			$this->error = $hrs['error'];
			return $hrs['return'];
		}

		if ($permanent) {
			$rs = $fm->deleteDir($path);
		} else {
			$trash = \FileRun::getTrash();
			$rs = $trash->store($relativePath);
		}

		if ($rs) {
			if ($permanent) {
				self::clearFolderData($path);
			} else {
				Share::clear($path);
				\FileRun::getWebLinks()->deleteAllByPath($path);
				Stars::removeByPath($path);
				Paths::updatePaths($path, $trash->lastStoredPath);//move paths
				Paths::moveFile($path, $trash->lastStoredPath);
			}

			Log::add($uid = false, "folder_deleted", array(
				"full_path" => $path,
				"relative_path" => $relativePath,
				"permanent" => $permanent
			));
		} else {
			Log::add($uid = false, "folder_deletion_failed", array(
				"full_path" => $path,
				"relative_path" => $relativePath,
				"permanent" => $permanent,
				"error" => $trash->error
			));
		}
		return $rs;
	}

	static function clearFolderData($path) {
		Share::clear($path);
		\FileRun::getWebLinks()->deleteAllByPath($path);
		Stars::removeByPath($path);

		SearchIndexQueue::add($path, "deleteFolder");
		MetaFiles::deleteByFolder($path);
		Paths::deletePath($path, true);
	}

	function moveFile($fromRelativePath, $toRelativePath) {
		global $fm;
		if (!$this->isValidRelativePath($fromRelativePath)) {
			return false;
		}
		if (!$this->isValidRelativePath($toRelativePath)) {
			return false;
		}

		$fromFilePath = $this->getUserAbsolutePath($fromRelativePath);
		$toFilePath = $this->getUserAbsolutePath($toRelativePath);
		$toPath = \FM::dirname($toFilePath);

		if (!is_file($fromFilePath)) {
			$this->error = array("code" => 6, "msg" => "Invalid source path, not a file.");
			return false;
		}
		if (!is_dir($toPath)) {
			$this->error = array("code" => 7, "msg" => "Invalid target path, not a folder.");
			return false;
		}
		if (file_exists($toFilePath)) {
			$this->error = array("code" => 8, "msg" => "Invalid target path, file already exists.");
			return false;
		} else {

			global $config;
			if (Versions::isLockedBySomebodyElse($fromFilePath)) {//check locking
				$this->error = array("code" => 9, "msg" => "File is locked by somebody else.");
				return false;
			}
			$filesize = filesize($fromFilePath);
			$rs = $fm->move($fromFilePath, $toFilePath, $config['system']['safe_file_move']);
			if ($rs) {
				@Thumbs\Utils::moveCacheItem($fromFilePath, $toFilePath);
				if (Paths::exists($fromFilePath)) {
					Paths::moveFile($fromFilePath, $toFilePath);
				}

				Versions::move($fromFilePath, $toFilePath);
				SearchIndexQueue::add($fromFilePath, "delete");
				SearchIndexQueue::add($toFilePath, "index");
				Share::rename($fromFilePath, $toFilePath);

				Log::add($uid = false, "file_moved", array(
					"from_full_path" => $fromFilePath,
					"from_relative_path" => $fromRelativePath,
					"new_full_path" => $toFilePath,
					"to_relative_path" => $toRelativePath,
					"filesize" => $filesize
				));

				//add also to "users in path" as a "received file"
				global $auth;
				$usersInPath = Utils::findUsersInPath($toPath);
				foreach ($usersInPath as $uid) {//remove current user if in list
					if ($uid != $auth->currentUserInfo['id']) {
						$receivingUserHomeFolder = $this->getUserAbsolutePath("/", $uid);
						$relPath = str_replace($receivingUserHomeFolder, "", $toFilePath);
						$relPath = gluePath("/ROOT/HOME/", $relPath);
						Log::add($uid, "receive_move", array(
							"from_uid" => $auth->currentUserInfo['id'],
							"from_full_path" => $fromFilePath,
							"from_relative_path" => $fromRelativePath,
							"to_full_path" => $toFilePath,
							"to_relative_path" => $toRelativePath,
							"to_user_relative_path" => $relPath,
							"filesize" => $filesize
						));
					}
				}

			} else {
				Log::add($uid = false, "file_move_failed", array(
					"from_full_path" => $fromFilePath,
					"from_relative_path" => $fromRelativePath,
					"new_full_path" => $toFilePath,
					"to_relative_path" => $toRelativePath,
					"filesize" => $filesize,
					"error" => $fm->errorMsg
				));
			}
			return $rs;
		}
	}


	function moveFolder($fromRelativePath, $toRelativePath) {
		global $fm;
		if (!$this->isValidRelativePath($fromRelativePath)) {
			return false;
		}
		if (!$this->isValidRelativePath($toRelativePath)) {
			return false;
		}

		$fromFilePath = $this->getUserAbsolutePath($fromRelativePath);
		$toFilePath = $this->getUserAbsolutePath($toRelativePath);
		$toPath = \FM::dirname($toFilePath);

		if (!is_dir($fromFilePath)) {
			$this->error = array("code" => 10, "msg" => "Invalid source path, not a folder.");
			return false;
		}
		if (!is_dir($toPath)) {
			$this->error = array("code" => 11, "msg" => "Invalid target path, not a folder.");
			return false;
		}

		if (file_exists($toFilePath)) {
			$this->error = array("code" => 12, "msg" => "Invalid target path, path already exists.");
			return false;
		} else {
			global $config;
			$rs = $fm->move($fromFilePath, $toFilePath, $config['system']['safe_file_move']);
			if ($rs) {

				Paths::updatePaths($fromFilePath, $toFilePath);
				Paths::updatePath($fromFilePath, $toFilePath);

				SearchIndexQueue::add($fromFilePath, "deleteFolder");
				SearchIndexQueue::add($toFilePath, "indexFolder");
				Share::rename($fromFilePath, $toFilePath);

				Log::add($uid = false, "folder_moved", array(
					"from_full_path" => $fromFilePath,
					"from_relative_path" => $fromRelativePath,
					"new_full_path" => $toFilePath,
					"to_relative_path" => $toRelativePath
				));

				//add also to "users in path" as a "received folder"
				$usersInPath = Utils::findUsersInPath($toFilePath);
				foreach ($usersInPath as $uid) {
					if ($uid != $this->uid) {//remove current user if in list
						$receivingUserHomeFolder = $this->getUserAbsolutePath("/", $uid);
						$relPath = str_replace($receivingUserHomeFolder, "", $toFilePath);
						$relPath = gluePath("/ROOT/HOME/", $relPath);
						Log::add($uid, "folder_receive_move", array(
							"from_uid" => $this->uid,
							"from_full_path" => $fromFilePath,
							"from_relative_path" => $fromRelativePath,
							"to_full_path" => $toFilePath,
							"to_relative_path" => $toRelativePath,
							"to_user_relative_path" => $relPath
						));
					}
				}
			} else {
				Log::add($uid = false, "folder_move_failed", array(
					"from_full_path" => $fromFilePath,
					"from_relative_path" => $fromRelativePath,
					"new_full_path" => $toFilePath,
					"to_relative_path" => $toRelativePath,
					"error" => $fm->errorMsg
				));
			}
			return $rs;
		}
	}

	function copyFile($fromRelativePath, $toRelativePath) {
		global $fm, $settings;
		$fromFilePath = $this->getUserAbsolutePath($fromRelativePath);
		$toFilePath = $this->getUserAbsolutePath($toRelativePath);
		$targetParentPath = \FM::dirname($toFilePath);

		if ($fromFilePath == $toFilePath) {
			$this->error = array("code" => 13, "msg" => "The destination path is the same as the source.");
			return false;
		}
		if (!is_file($fromFilePath)) {
			$this->error = array("code" => 14, "msg" => "Invalid source, not a file.");
			return false;
		}
		if (!is_dir($targetParentPath)) {
			$this->error = array("code" => 15, "msg" => "Invalid target, not a folder.");
			return false;
		}
		if (file_exists($toFilePath)) {
			if (!is_file($toFilePath)) {
				$this->error = array("code" => 16, "msg" => "Destination target is a folder.");
				return false;
			}
			if (!$settings->versioning_max) {
				$this->error = array("code" => 17, "msg" => "Invalid target, already exists.");
				return false;
			} else {
				//overwriting (and adding new version)
				if (Versions::isLockedBySomebodyElse($toFilePath)) {//check locking
					$this->error = array("code" => 18, "msg" => "File is locked by somebody else.");
					return false;
				}
				$rs = Versions::add($toFilePath, array("user" => Users::getNameById($this->uid)));//add the current version to repository
				if ($rs) {
					$rs = unlink($toFilePath);//delete the current version, to be replaced
					if (!$rs) {
						$this->error = array("code" => 19, "msg" => "Failed to delete file after storing its current version.");
						return false;
					}
				} else {
					$this->error = array("code" => 20, "msg" => "Failed to add existing file version to repository.");
					return false;
				}
			}
		}
		$rs = $fm->copyFile($fromFilePath, $toFilePath);
		if ($rs) {
			Paths::addIfNotFound($toFilePath);

			SearchIndexQueue::add($toFilePath, "index", $this->uid);
			MetaTypes::auto($toFilePath, $this->uid);

			Log::add($uid = false, "file_copied", array(
				"from_full_path" => $fromFilePath,
				"from_relative_path" => $fromRelativePath,
				"to_full_path" => $toFilePath,
				"to_relative_path" => $toRelativePath
			));

			//add also to "users in path" as a "received file"
			$usersInPath = Utils::findUsersInPath($toFilePath);
			foreach ($usersInPath as $uid) {//remove current user if in list
				if ($uid != $this->uid) {
					$receivingUserHomeFolder = $this->getUserAbsolutePath("/", $uid);
					$relPath = str_replace($receivingUserHomeFolder, "", $toFilePath);
					$relPath = gluePath("/ROOT/HOME/", $relPath);
					Log::add($uid, "receive_copy", array(
						"from_uid" => $this->uid,
						"from_full_path" => $fromFilePath,
						"from_relative_path" => $fromRelativePath,
						"to_full_path" => $toFilePath,
						"to_relative_path" => $toRelativePath,
						"to_user_relative_path" => $relPath
					));
				}
			}
		} else {
			Log::add($uid = false, "file_copy_failed", array(
				"from_full_path" => $fromFilePath,
				"from_relative_path" => $fromRelativePath,
				"to_full_path" => $toFilePath,
				"to_relative_path" => $toRelativePath,
				"error" => $fm->errorMsg
			));
		}
		return $rs;
	}

	function copyFolder($relativePathFrom, $targetRelativePath) {
		global $fm;

		if (!$this->isValidRelativePath($relativePathFrom)) {
			return false;
		}
		if (!$this->isValidRelativePath($targetRelativePath)) {
			return false;
		}

		$fromPath = $this->getUserAbsolutePath($relativePathFrom);
		$toPath = $this->getUserAbsolutePath($targetRelativePath);

		if (!is_dir($fromPath)) {
			$this->error = array("code" => 21, "msg" => "Invalid source path, not a folder.");
			return false;
		}
		if (!is_dir(\FM::dirname($toPath))) {
			$this->error = array("code" => 22, "msg" => "Invalid target path, not a folder.");
			return false;
		}

		if (is_dir($toPath)) {
			$this->error = array("code" => 23, "msg" => "Invalid target path, path already exists.");
			return false;
		} else {
			$rs = $fm->copyDir($fromPath, $toPath, function($oldPath, $newPath) {
				Paths::addIfNotFound($newPath);
				MetaTypes::auto($newPath);
			});
			$logData = array(
				"from_full_path" => $fromPath,
				"from_relative_path" => $relativePathFrom,
				"new_full_path" => $toPath,
				"to_relative_path" => $targetRelativePath
			);

			if ($rs) {
				SearchIndexQueue::add($toPath, "indexFolder");//queue for indexing
				Log::add($uid = false, "folder_copied", $logData);

				//add also to "users in path" as a "received folder"
				$usersInPath = Utils::findUsersInPath($toPath);
				foreach ($usersInPath as $uid) {
					if ($uid != $this->uid) {//remove current user if in list
						$receivingUserHomeFolder = $this->getUserAbsolutePath("/", $uid);
						$relPath = str_replace($receivingUserHomeFolder, "", $toPath);
						$relPath = gluePath("/ROOT/HOME/", $relPath);
						Log::add($uid, "folder_receive_copy", array(
							"from_uid" => $this->uid,
							"from_full_path" => $fromPath,
							"from_relative_path" => $relativePathFrom,
							"to_full_path" => $toPath,
							"to_relative_path" => $targetRelativePath,
							"to_user_relative_path" => $relPath
						));
					}
				}
			} else {
				Log::add($uid = false, "folder_copy_failed", $logData);
			}
			return $rs;
		}
	}

	function newFile($relativePath, $filename, $src = false, $contents = false, $move = false) {
		global $fm, $settings;
		$this->error = false;

		if (!$this->isValidRelativePath($relativePath)) {
			return false;
		}
		clearstatcache();

		$path = $this->getUserAbsolutePath($relativePath);
		$destPath = gluePath($path, $filename);
		if (strlen($src) > 0) {
			$filesize = filesize($src);
		} else {
			$filesize = strlen($contents);
		}

		$logDetails = array(
			"full_path" => $destPath,
			"relative_path" => $relativePath,
			"filename" => $filename,
			"filesize" => $filesize
		);

		$ext = \FM::getExtension($filename);
		$blockedTypes = $settings->upload_blocked_types;
		if (strlen($blockedTypes) > 0) {
			$blockedTypes = explode(',', strtolower($blockedTypes));
			$blockedTypes = array_map('trim', $blockedTypes);
			if (in_array($ext, $blockedTypes)) {
				$this->error = array("code" => 24, "msg" => 'The file type is not allowed.');
			}
		}

		$fileExists = file_exists($destPath);
		if ($fileExists) {
			if (Versions::isLockedBySomebodyElse($destPath, $this->uid)) {//check locking
				$this->error = array("code" => 25, "msg" => "File is locked by somebody else.");
			}
		}

		if (!$this->error) {
			$hrs = \FileRun::hook('file.upload', func_get_args(), array('filePath' => $destPath));
			if (is_array($hrs)) {
				if ($hrs['error']) {
					$this->error = $hrs['error'];
				}
			}
		}

		if (!$this->error) {
			if ($fileExists) {
				if (!$settings->versioning_max || \FM::getFileSize($destPath) == 0) {
					$rs = unlink($destPath);
					if (!$rs) {
						$this->error = array("code" => 43, "msg" => "Failed to delete the empty existing file.");
					}
				} else {
					//add the current version to repository
					$rs = Versions::add($destPath, array("user" => Users::getNameById($this->uid)));
					if ($rs) {
						//delete the current version, to be replaced
						$rs = @unlink($destPath);
						if (!$rs) {
							$this->error = array("code" => 42, "msg" => "Failed to delete the current version.");
						}
					} else {
						$this->error = array("code" => 41, "msg" => "Failed to add current file to version repository.");
					}
				}
			}
		}

		if (!$this->error) {
			if ($move) {
				$rs = @rename($src, $destPath);
				if (!$rs) {
					$this->error = array("code" => 35, "msg" => 'Failed to rename ' . $src . ' to ' . $destPath);
				}
			} else {
				if ($src) {
					if ($src != $destPath) {
						$rs = $fm->copyFile($src, $destPath);
						if (!$rs) {
							$this->error = array("code" => 26, "msg" => "File copy failed: " . $fm->errorMsg);
						}
					}
				} else {
					$rs = $fm->newFile($destPath, $contents);
					if (!$rs) {
						$this->error = array("code" => 28, "msg" => "File creation failed: " . $fm->errorMsg);
					}
				}
			}
		}

		if (!$this->error) {
			Log::add($this->uid, "upload", $logDetails);
			Paths::addIfNotFound($destPath);
			SearchIndexQueue::add($destPath, "index", $this->uid);
			if (!$fileExists) {
				MetaTypes::auto($destPath, $this->uid);
			}
			//log also for receiving users
			$usersInPath = Utils::findUsersInPath($path);
			foreach ($usersInPath as $uid) {
				if ($uid != $this->uid) {
					$receivingUserHomeFolder = $this->getUserAbsolutePath('/', $uid);
					$relPath = str_replace($receivingUserHomeFolder, '', $destPath);
					$relPath = gluePath("/ROOT/HOME/", $relPath);
					Log::add($uid, "receive_upload", array(
						"from_uid" => $this->uid,
						"to_full_path" => $destPath,
						"to_relative_path" => $relativePath,
						"to_user_relative_path" => $relPath
					));
				}
			}

			return true;
		}

		$logDetails['error'] = $this->error['msg'];
		Log::add($this->uid, 'upload_failed', $logDetails);
		return false;
	}

	function rename($relativePath, $newFileName) {
		global $fm, $settings;
		$this->error = false;
		if (!$this->isValidRelativePath($relativePath)) {
			return false;
		}
		$path = $this->getUserAbsolutePath($relativePath);
		$parentPath = \FM::dirname($path);
		$newPath = gluePath($parentPath, $newFileName);

		$oldName = \FM::basename($path);
		$changeCase = (strtolower($oldName) == strtolower($newFileName) && ($oldName != $newFileName));

		$logDetails = array(
			"full_path" => $newPath,
			"relative_path" => $relativePath,
			"old_filename" => $oldName
		);

		if (file_exists($newPath) && !$changeCase) {
			$this->error = array("code" => 36, "msg" => "There is already a file or a folder with the same name!");
		}

		if (!$this->error) {
			if (Versions::isLockedBySomebodyElse($path)) {
				$this->error = array("code" => 37, "msg" => "File is locked by somebody else.");
			}
		}

		$resourceType = is_file($path) ? "file" : "folder";

		if (!$this->error) {
			if ($resourceType == 'file') {
				$ext = \FM::getExtension($newFileName);
				$blockedTypes = $settings->upload_blocked_types;
				if (strlen($blockedTypes) > 0) {
					$blockedTypes = explode(',', strtolower($blockedTypes));
					$blockedTypes = array_map('trim', $blockedTypes);
					if (in_array($ext, $blockedTypes)) {
						$this->error = array("code" => 44, "msg" => 'The file type is not allowed.');
					}
				}
			}
		}

		if (!$this->error) {
			$rs = $fm->rename($path, $newFileName);
			if (!$rs) {
				$this->error = array("code" => 26, "msg" => "Rename failed: " . $fm->errorMsg);
			}
			if (!$this->error) {
				if ($resourceType == 'folder') {
					Paths::updatePaths($path, $newPath);
				} else {
					Thumbs\Utils::renameCacheItem($path, $newFileName);
				}
				Paths::renameItem($path, $newFileName);
				Paths::addIfNotFound($newPath);

				SearchIndexQueue::add($path, ($resourceType == "file") ? "delete" : "deleteFolder");
				SearchIndexQueue::add($newPath, ($resourceType == "file") ? "index" : "indexFolder");
				Versions::rename($path, $newFileName);
				Share::rename($path, $newPath);

				Log::add($this->uid, $resourceType."_renamed", $logDetails);
				return true;
			}
		}

		$logDetails['error'] = $this->error['msg'];
		Log::add($this->uid, "failed_".$resourceType."_rename", $logDetails);

		return false;
	}

	function createPath($relativePath, $newFileName = '') {
		global $fm;
		$this->error = false;
		if (!$this->isValidRelativePath($relativePath)) {
			return false;
		}

		$path = $this->getUserAbsolutePath($relativePath);
		$newPath = gluePath($path, $newFileName);

		$logDetails = array(
			"full_path" => $newPath,
			"relative_path" => gluePath($relativePath, $newFileName)
		);

		if (is_dir($newPath)) {
			$this->error = array("code" => 39, "msg" => 'There is already a folder with the same name in this folder!');
		}
		if (is_file($newPath)) {
			$this->error = array("code" => 45, "msg" => 'There is already a file with the same name in this folder!');
		}

		if (!$this->error) {
			$rs = $fm->createPath($newPath);
			if (!$rs) {
				$this->error = array("code" => 40, "msg" => "Failed to create path: " . $fm->errorMsg);
			}
		}
		if (!$this->error) {
			Log::add($this->uid, "new_folder", $logDetails);
			Paths::addIfNotFound($newPath);

			return true;
		}

		$logDetails['error'] = $this->error['msg'];
		Log::add($this->uid, "new_folder_failed", $logDetails);

		return false;
	}
}